@@ -647,7 +647,7 @@ htp_status_t htp_connp_REQ_HEADERS(htp_connp_t *connp) {
             #endif           
 
             // Should we terminate headers?
-            if (htp_connp_is_line_terminator(connp, data, len)) {
+            if (htp_connp_is_line_terminator(connp, data, len, 0)) {
                 // Parse previous header, if any.
                 if (connp->in_header != NULL) {
                     if (connp->cfg->process_request_header(connp, bstr_ptr(connp->in_header),
@@ -816,6 +816,10 @@ htp_status_t htp_connp_REQ_LINE_complete(htp_connp_t *connp) {
 htp_status_t htp_connp_REQ_LINE(htp_connp_t *connp) {
     for (;;) {
         // Get one byte
+        IN_PEEK_NEXT(connp);
+        if (connp->in_status == HTP_STREAM_CLOSED && connp->in_next_byte == -1) {
+            return htp_connp_REQ_LINE_complete(connp);
+        }
         IN_COPY_BYTE_OR_RETURN(connp);
 
         // Have we reached the end of the line?
@@ -869,7 +873,12 @@ htp_status_t htp_connp_REQ_FINALIZE(htp_connp_t *connp) {
     while ((pos < len) && (!htp_is_space(data[pos])))
         pos++;
 
-    if (pos > mstart) {
+    if (pos <= mstart) {
+        //empty whitespace line
+        htp_status_t rc = htp_tx_req_process_body_data_ex(connp->in_tx, data, len);
+        htp_connp_req_clear_buffer(connp);
+        return rc;
+    } else {
         int methodi = HTP_M_UNKNOWN;
         bstr *method = bstr_dup_mem(data + mstart, pos - mstart);
         if (method) {
@@ -882,9 +891,8 @@ htp_status_t htp_connp_REQ_FINALIZE(htp_connp_t *connp) {
             htp_status_t rc = htp_tx_req_process_body_data_ex(connp->in_tx, data, len);
             htp_connp_req_clear_buffer(connp);
             return rc;
-        }
+        } // else continue
     }
-    //else
     //unread last end of line so that REQ_LINE works
     if (connp->in_current_read_offset < (int64_t)len) {
         connp->in_current_read_offset=0;
