@@ -464,7 +464,7 @@ int htp_is_folding_char(int c) {
  * @param[in] len
  * @return 0 or 1
  */
-int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t len) {
+int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t len, int next_no_lf) {
     // Is this the end of request headers?
     switch (connp->cfg->server_personality) {
         case HTP_SERVER_IIS_5_1:
@@ -481,10 +481,7 @@ int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t
             }
             // Only space is terminator if terminator does not follow right away
             if (len == 2 && htp_is_lws(data[0]) && data[1] == LF) {
-                if (connp->out_current_read_offset < connp->out_current_len &&
-                    connp->out_current_data[connp->out_current_read_offset] != LF) {
-                    return 1;
-                }
+                return next_no_lf;
             }
             break;
     }
@@ -501,7 +498,7 @@ int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t
  * @return 0 or 1
  */
 int htp_connp_is_line_ignorable(htp_connp_t *connp, unsigned char *data, size_t len) {
-    return htp_connp_is_line_terminator(connp, data, len);
+    return htp_connp_is_line_terminator(connp, data, len, 0);
 }
 
 static htp_status_t htp_parse_port(unsigned char *data, size_t len, int *port, int *invalid) {
@@ -1860,136 +1857,6 @@ bstr *htp_normalize_hostname_inplace(bstr *hostname) {
     return hostname;
 }
 
-#if 0
-
-/**
- * Replace the URI in the structure with the one provided as the parameter
- * to this function (which will typically be supplied in a Host header).
- *
- * @param[in] connp
- * @param[in] parsed_uri
- * @param[in] hostname
- */
-void htp_replace_hostname(htp_connp_t *connp, htp_uri_t *parsed_uri, bstr *hostname) {
-    if (hostname == NULL) return;
-
-    bstr *new_hostname = NULL;
-
-    int colon = bstr_chr(hostname, ':');
-    if (colon == -1) {
-        // Hostname alone (no port information)
-        new_hostname = bstr_dup(hostname);
-        if (new_hostname == NULL) return;
-        htp_normalize_hostname_inplace(new_hostname);
-
-        if (parsed_uri->hostname != NULL) bstr_free(parsed_uri->hostname);
-        parsed_uri->hostname = new_hostname;
-    } else {
-        // Hostname and port
-        new_hostname = bstr_dup_ex(hostname, 0, colon);
-        if (new_hostname == NULL) return;
-        // TODO Handle whitespace around hostname
-        htp_normalize_hostname_inplace(new_hostname);
-
-        if (parsed_uri->hostname != NULL) bstr_free(parsed_uri->hostname);
-        parsed_uri->hostname = new_hostname;
-        parsed_uri->port_number = 0;
-
-        // Port
-        int port = htp_parse_positive_integer_whitespace((unsigned char *) bstr_ptr(hostname) + colon + 1,
-                bstr_len(hostname) - colon - 1, 10);
-        if (port < 0) {
-            // Failed to parse port
-            htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Invalid server port information in request");
-        } else if ((port > 0) && (port < 65536)) {
-            // Valid port
-            if ((connp->conn->server_port != 0) && (port != connp->conn->server_port)) {
-                // Port was specified in connection and is different from the TCP port
-                htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Request server port=%d number differs from the actual TCP port=%d", port, connp->conn->server_port);
-            } else {
-                parsed_uri->port_number = port;
-            }
-        }
-    }
-}
-
-/**
- * Is URI character reserved?
- *
- * @param[in] c
- * @return 1 if it is, 0 if it isn't
- */
-int htp_is_uri_unreserved(unsigned char c) {
-    if (((c >= 0x41) && (c <= 0x5a)) ||
-            ((c >= 0x61) && (c <= 0x7a)) ||
-            ((c >= 0x30) && (c <= 0x39)) ||
-            (c == 0x2d) || (c == 0x2e) ||
-            (c == 0x5f) || (c == 0x7e)) {
-        return 1;
-    } else {
-        return 0;
-    }
-}
-
-/**
- * Decode a URL-encoded string, leaving the reserved
- * characters and invalid encodings alone.
- *
- * @param[in] s
- */
-void htp_uriencoding_normalize_inplace(bstr *s) {
-    if (s == NULL) return;
-
-    unsigned char *data = bstr_ptr(s);
-    if (data == NULL) return;
-    size_t len = bstr_len(s);
-
-    size_t rpos = 0;
-    size_t wpos = 0;
-
-    while (rpos < len) {
-        if (data[rpos] == '%') {
-            if (rpos + 2 < len) {
-                if (isxdigit(data[rpos + 1]) && (isxdigit(data[rpos + 2]))) {
-                    unsigned char c = x2c(&data[rpos + 1]);
-
-                    if (htp_is_uri_unreserved(c)) {
-                        // Leave reserved characters encoded, but convert
-                        // the hexadecimal digits to uppercase
-                        data[wpos++] = data[rpos++];
-                        data[wpos++] = toupper(data[rpos++]);
-                        data[wpos++] = toupper(data[rpos++]);
-                    } else {
-                        // Decode unreserved character
-                        data[wpos++] = c;
-                        rpos += 3;
-                    }
-                } else {
-                    // Invalid URL encoding: invalid hex digits
-
-                    // Copy over what's there
-                    data[wpos++] = data[rpos++];
-                    data[wpos++] = toupper(data[rpos++]);
-                    data[wpos++] = toupper(data[rpos++]);
-                }
-            } else {
-                // Invalid URL encoding: string too short
-
-                // Copy over what's there
-                data[wpos++] = data[rpos++];
-                while (rpos < len) {
-                    data[wpos++] = toupper(data[rpos++]);
-                }
-            }
-        } else {
-            data[wpos++] = data[rpos++];
-        }
-    }
-
-    bstr_adjust_len(s, wpos);
-}
-#endif
-
 /**
  * Normalize URL path. This function implements the remove dot segments algorithm
  * specified in RFC 3986, section 5.2.4.
@@ -2360,21 +2227,6 @@ bstr *htp_unparse_uri_noencode(htp_uri_t *uri) {
     if (uri->query != NULL) {
         bstr_add_c_noex(r, "?");
         bstr_add_noex(r, uri->query);
-
-        /*
-        bstr *query = bstr_dup(uri->query);
-        if (query == NULL) {
-            bstr_free(r);
-            return NULL;
-        }
-
-        htp_uriencoding_normalize_inplace(query);
-
-        bstr_add_c_noex(r, "?");
-        bstr_add_noex(r, query);
-
-        bstr_free(query);
-        */
     }
 
     if (uri->fragment != NULL) {
